import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { Inject } from '@nestjs/common';
import { AggregateID } from '@mobicoop/ddd-library';
import { AUTHENTICATION_REPOSITORY } from '@modules/authentication/authentication.di-tokens';
import { UpdatePasswordCommand } from './update-password.command';
import { AuthenticationRepositoryPort } from '../../ports/authentication.repository.port';
import { AuthenticationEntity } from '@modules/authentication/core/domain/authentication.entity';

@CommandHandler(UpdatePasswordCommand)
export class UpdatePasswordService implements ICommandHandler {
  constructor(
    @Inject(AUTHENTICATION_REPOSITORY)
    private readonly authenticationRepository: AuthenticationRepositoryPort,
  ) {}

  async execute(command: UpdatePasswordCommand): Promise<AggregateID> {
    const authentication: AuthenticationEntity =
      await this.authenticationRepository.findOneById(command.userId);
    await authentication.updatePassword(command.password);
    await this.authenticationRepository.update(command.userId, authentication);
    return authentication.id;
  }
}
