import { Type } from '@modules/authentication/core/domain/username.types';
import {
  registerDecorator,
  ValidationOptions,
  ValidationArguments,
  isEmail,
  isPhoneNumber,
} from 'class-validator';

export function IsValidUsername(validationOptions?: ValidationOptions) {
  return function (object: any, propertyName: string) {
    registerDecorator({
      name: 'isValidUsername',
      target: object.constructor,
      propertyName: propertyName,
      options: validationOptions,
      validator: {
        validate(value: any, args: ValidationArguments) {
          const usernameType: Type = args.object['type'];
          switch (usernameType) {
            case Type.PHONE:
              return isPhoneNumber(value);
            case Type.EMAIL:
              return isEmail(value);
            default:
              return false;
          }
        },
      },
    });
  };
}
