import { Inject, Injectable } from '@nestjs/common';
import {
  HealthCheckError,
  HealthCheckResult,
  HealthIndicator,
  HealthIndicatorResult,
} from '@nestjs/terminus';
import { CheckRepositoryPort } from '../ports/check-repository.port';
import {
  AUTHENTICATION_REPOSITORY,
  USERNAME_REPOSITORY,
} from '@modules/health/health.di-tokens';
import { MESSAGE_PUBLISHER } from '@src/app.di-tokens';
import { LOGGING_AUTHENTICATION_HEALTH_CRIT } from '@modules/health/health.constants';
import { MessagePublisherPort } from '@mobicoop/ddd-library';
import { AuthenticationRepositoryPort } from '@modules/authentication/core/application/ports/authentication.repository.port';
import { UsernameRepositoryPort } from '@modules/authentication/core/application/ports/username.repository.port';

@Injectable()
export class RepositoriesHealthIndicatorUseCase extends HealthIndicator {
  private _checkRepositories: CheckRepositoryPort[];
  constructor(
    @Inject(AUTHENTICATION_REPOSITORY)
    private readonly authenticationRepository: AuthenticationRepositoryPort,
    @Inject(USERNAME_REPOSITORY)
    private readonly usernameRepository: UsernameRepositoryPort,
    @Inject(MESSAGE_PUBLISHER)
    private readonly messagePublisher: MessagePublisherPort,
  ) {
    super();
    this._checkRepositories = [authenticationRepository, usernameRepository];
  }
  isHealthy = async (key: string): Promise<HealthIndicatorResult> => {
    try {
      await Promise.all(
        this._checkRepositories.map(
          async (checkRepository: CheckRepositoryPort) => {
            await checkRepository.healthCheck();
          },
        ),
      );
      return this.getStatus(key, true);
    } catch (error) {
      const healthCheckResult: HealthCheckResult = error;
      this.messagePublisher.publish(
        LOGGING_AUTHENTICATION_HEALTH_CRIT,
        JSON.stringify(healthCheckResult.error),
      );
      throw new HealthCheckError('Repository', {
        repository: error.message,
      });
    }
  };
}
