import { Module, Provider } from '@nestjs/common';
import { HealthHttpController } from './interface/http-controllers/health.http.controller';
import { TerminusModule } from '@nestjs/terminus';
import { RepositoriesHealthIndicatorUseCase } from './core/application/usecases/repositories.health-indicator.usecase';
import { HealthGrpcController } from './interface/grpc-controllers/health.grpc.controller';
import { MessageBrokerPublisher } from '@mobicoop/message-broker-module';
import { AuthenticationRepository } from '@modules/authentication/infrastructure/authentication.repository';
import {
  AUTHENTICATION_REPOSITORY,
  USERNAME_REPOSITORY,
} from './health.di-tokens';
import { UsernameRepository } from '@modules/authentication/infrastructure/username.repository';
import { MESSAGE_PUBLISHER } from '@src/app.di-tokens';
import { AuthenticationModule } from '@modules/authentication/authentication.module';

const grpcControllers = [HealthGrpcController];

const httpControllers = [HealthHttpController];

const useCases: Provider[] = [RepositoriesHealthIndicatorUseCase];

const repositories: Provider[] = [
  {
    provide: AUTHENTICATION_REPOSITORY,
    useClass: AuthenticationRepository,
  },
  {
    provide: USERNAME_REPOSITORY,
    useClass: UsernameRepository,
  },
];

const messageBrokers: Provider[] = [
  {
    provide: MESSAGE_PUBLISHER,
    useClass: MessageBrokerPublisher,
  },
];

@Module({
  imports: [TerminusModule, AuthenticationModule],
  controllers: [...grpcControllers, ...httpControllers],
  providers: [...useCases, ...repositories, ...messageBrokers],
})
export class HealthModule {}
